package be.iminds.ilabt.jfed.experimenter_gui.editor.properties;

import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.experimenter_gui.util.BrowserUtil;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.AdvertisementCounts;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.ServerGlimpse;
import javafx.fxml.FXML;
import javafx.scene.Parent;
import javafx.scene.control.Hyperlink;
import javafx.scene.control.Label;
import javafx.scene.control.ListCell;
import javafx.scene.layout.*;

import javax.annotation.Nonnull;
import java.util.Objects;
import java.util.Optional;

/**
 * HardwareTypeListCell
 */
public class HardwareTypeListCell extends ListCell<JFedGuiConfig.HardwareTypeInfo> {
    @Nonnull private final JFedGuiConfig config;
    @Nonnull private final Server server;
    @Nonnull private final BrowserUtil browserUtil;

    protected final VBox container = new VBox(2);

    private final Label nameLabel;
    private final Label countLabel;
    private final Label infoLabel;
    private final Hyperlink infoLink;

    public HardwareTypeListCell(@Nonnull JFedGuiConfig config,
                                @Nonnull Server server,
                                @Nonnull BrowserUtil browserUtil) {
        this.config = config;
        this.server = server;
        this.browserUtil = browserUtil;

        nameLabel = new Label();
        countLabel = new Label();
        infoLabel = new Label();
        infoLink = new Hyperlink();

        infoLabel.managedProperty().bind(infoLabel.visibleProperty());
        infoLink.managedProperty().bind(infoLink.visibleProperty());
        countLabel.managedProperty().bind(countLabel.visibleProperty());
        Region filler = new Region();
        HBox.setHgrow(filler, Priority.ALWAYS);

        HBox container2 = new HBox(2);

        container2.getChildren().addAll(nameLabel, filler, countLabel);
        container.getChildren().addAll(container2, infoLabel/*, infoLink*/);

        infoLabel.setWrapText(true);
        infoLabel.setMaxWidth(400);
        infoLabel.setPrefWidth(330);
        infoLabel.setStyle("-fx-word-wrap: break-word; -fx-font-scale: 75%; -fx-text-fill: darkgrey");

        setGraphic(container);
    }

    @Override
    protected void updateItem(JFedGuiConfig.HardwareTypeInfo item, boolean empty) {
        super.updateItem(item, empty);

        if (item == null) {
            setText(null);
            setGraphic(null);
            return;
        }

        setText(null);
        setGraphic(container);

        nameLabel.setText(item.getHardwareTypeName());
        countLabel.setText("?/?");

        boolean hasAnyInfo =  item.getHardwareTypeName() != null &&
                config.getHardwareTypeInfo(server).get() != null &&
//                config.getHardwareTypeInfo(server).get().getEntries() != null &&
                !config.getHardwareTypeInfo(server).get().getEntries().isEmpty();

        boolean showInfo = false;
        boolean showLink = false;

        if (hasAnyInfo) {
            Optional<ServerGlimpse.AdvertisementInfo.HardwareTypeInfo.HardwareTypeInfoEntry> hwTypeInfo =
                    config.getHardwareTypeInfo(server).get()
                            .getEntries().stream()
                            .filter(e -> Objects.equals(e.getName(), item.getHardwareTypeName()))
                            .findAny();

            if (hwTypeInfo.isPresent()) {
                if (hwTypeInfo.get().getHrn() != null) {
                    showInfo = true;
                    infoLabel.setText(hwTypeInfo.get().getHrn());
                } else {
                    if (hwTypeInfo.get().getTextDescription() != null) {
                        showInfo = true;
                        infoLabel.setText(hwTypeInfo.get().getTextDescription());
                    }
                }
                if (hwTypeInfo.get().getHtmlDescription() != null) {
                    showLink = true;
                    infoLink.setOnAction(event -> this.browserUtil.openUrlInBrowser(hwTypeInfo.get().getHtmlDescription()));
                }
            }

            AdvertisementCounts counts = config.getServerNodeCounts(server).get();
            if (counts == null) {
                countLabel.setVisible(false);
            } else {
                String hwType = item.getHardwareTypeName();
                Integer allCount = counts.getHwCount(hwType, AdvertisementCounts.CountScope.ALL);
                Integer availCount = counts.getHwCount(hwType, AdvertisementCounts.CountScope.AVAILABLE);
                if (allCount != null || availCount != null) {
                    countLabel.setText(String.format("%d/%d", availCount, allCount));
                    countLabel.setVisible(true);
                } else {
                    countLabel.setVisible(false);
                }
            }
        }

        infoLink.setVisible(showLink);
        infoLabel.setVisible(showInfo);
    }
}
